@@ -34,7 +34,6 @@
 #include <openais/totem/totemip.h>
 #include <openais/totem/totempg.h>
 #include <openais/service/swab.h>
-#include <openais/service/print.h>
 #include <openais/service/timer.h>
 #include <openais/totem/aispoll.h>
 #include "list.h"
@@ -48,6 +47,7 @@
 #include "ais.h"
 
 #define max(a,b) (((a) > (b)) ? (a) : (b))
+LOGSYS_DECLARE_SUBSYS (CMAN_NAME, LOG_INFO);
 
 /* Reference counting for cluster applications */
 static int use_count;
@@ -96,7 +96,7 @@ static int get_highest_nodeid(void);
 static int send_port_open_msg(unsigned char port);
 static int send_port_enquire(int nodeid);
 static void process_internal_message(char *data, int len, int nodeid, int byteswap);
-static void recalculate_quorum(int allow_decrease);
+static void recalculate_quorum(int allow_decrease, int by_current_nodes);
 static void send_kill(int nodeid, uint16_t reason);
 static char *killmsg_reason(int reason);
 
@@ -159,9 +159,9 @@ static void set_quorate(int total_votes)
 	}
 
 	if (cluster_is_quorate && !quorate)
-		log_msg(LOG_INFO, "quorum lost, blocking activity\n");
+		log_printf(LOG_INFO, "quorum lost, blocking activity\n");
 	if (!cluster_is_quorate && quorate)
-		log_msg(LOG_INFO, "quorum regained, resuming activity\n");
+		log_printf(LOG_INFO, "quorum regained, resuming activity\n");
 
 	/* If we are newly quorate, then kill any AISONLY nodes */
 	if (!cluster_is_quorate && quorate) {
@@ -327,11 +327,11 @@ static int calculate_quorum(int allow_decrease, int max_expected, unsigned int *
 }
 
 /* Recalculate cluster quorum, set quorate and notify changes */
-static void recalculate_quorum(int allow_decrease)
+static void recalculate_quorum(int allow_decrease, int by_current_nodes)
 {
 	unsigned int total_votes;
 
-	quorum = calculate_quorum(allow_decrease, 0, &total_votes);
+	quorum = calculate_quorum(allow_decrease, by_current_nodes?cluster_members:0, &total_votes);
 	set_quorate(total_votes);
 	notify_listeners(NULL, EVENT_REASON_STATECHANGE, 0);
 }
@@ -344,7 +344,8 @@ static void copy_to_usernode(struct cluster_node *node,
 	int addrlen=0;
 	unsigned int numaddrs=1;
 	char **status;
-	struct totem_ip_address node_ifs[num_interfaces];
+	struct totem_ip_address node_ifs[INTERFACE_MAX];
+	/* totempg_ifaces_get always copies INTERFACE_MAX addresses */
 
 	strcpy(unode->name, node->name);
 	unode->jointime = node->join_time;
@@ -360,7 +361,6 @@ static void copy_to_usernode(struct cluster_node *node,
 	   must ask for them */
 	totempg_ifaces_get(node->node_id, node_ifs, &status, &numaddrs);
 
-
 	totemip_totemip_to_sockaddr_convert(&node_ifs[0], 0, &ss, &addrlen);
 	memcpy(unode->addr, &ss, addrlen);
 	unode->addrlen = addrlen;
@@ -381,9 +381,6 @@ int cman_set_nodeid(int nodeid)
 	if (ais_running)
 		return -EALREADY;
 
-	if (nodeid < 0 || nodeid > 4096)
-		return -EINVAL;
-
 	wanted_nodeid = nodeid;
 	return 0;
 }
@@ -404,7 +401,7 @@ int cman_join_cluster(char *name, unsigned short cl_id, int two_node_flag, int e
 	ais_running = 1;
 
 	if (read_ccs_nodes(&config_version, 1)) {
-		log_msg(LOG_ERR, "Can't initialise list of nodes from CCS\n");
+		log_printf(LOG_ERR, "Can't initialise list of nodes from CCS\n");
 		return -EINVAL;
 	}
 
@@ -416,8 +413,8 @@ int cman_join_cluster(char *name, unsigned short cl_id, int two_node_flag, int e
 	}
 
 	time(&join_time);
-	us = add_new_node(nodename, wanted_nodeid, -1, expected_votes,
-			  NODESTATE_MEMBER);
+	us = add_new_node(nodename, wanted_nodeid, 1, expected_votes,
+			  NODESTATE_DEAD);
 	set_port_bit(us, 0);
 	us->us = 1;
 
@@ -441,9 +438,8 @@ static int do_cmd_set_version(char *cmdbuf, int *retlen)
 	if (config_version == version->config)
 		return 0;
 
-	config_version = version->config;
 	/* We will re-read CCS when we get our own message back */
-	send_reconfigure(us->node_id, RECONFIG_PARAM_CONFIG_VERSION, config_version);
+	send_reconfigure(us->node_id, RECONFIG_PARAM_CONFIG_VERSION, version->config);
 	return 0;
 }
 
@@ -502,6 +498,8 @@ static int do_cmd_get_extrainfo(char *cmdbuf, char **retbuf, int retsize, int *r
 		einfo->flags |= CMAN_EXTRA_FLAG_SHUTDOWN;
 	if (uncounted)
 		einfo->flags |= CMAN_EXTRA_FLAG_UNCOUNTED;
+	if (us->flags & NODE_FLAGS_DIRTY)
+		einfo->flags |= CMAN_EXTRA_FLAG_DIRTY;
 
 	ptr = einfo->addresses;
 	for (i=0; i<num_interfaces; i++) {
@@ -596,17 +594,25 @@ static int do_cmd_get_node(char *cmdbuf, char *retbuf, int *retlen)
 	if (!we_are_a_cluster_member)
 		return -ENOENT;
 
-	if (!u_node->name[0]) {
-		if (u_node->node_id == 0)
-			u_node->node_id = us->node_id;
-		node = find_node_by_nodeid(u_node->node_id);
+	if (u_node->node_id == CLUSTER_GETNODE_QUORUMDEV) {
+		if (quorum_device)
+			node = quorum_device;
+		else
+			return -ENOENT;
 	}
-	else
-		node = find_node_by_name(u_node->name);
+	else {
+		if (!u_node->name[0]) {
+			if (u_node->node_id == 0)
+				u_node->node_id = us->node_id;
+			node = find_node_by_nodeid(u_node->node_id);
+		}
+		else
+			node = find_node_by_name(u_node->name);
 
-	if (!node) {
-		P_MEMB("cmd_get_node failed: id=%d, name='%s'\n", u_node->node_id, u_node->name);
-		return -ENOENT;
+		if (!node) {
+			P_MEMB("cmd_get_node failed: id=%d, name='%s'\n", u_node->node_id, u_node->name);
+			return -ENOENT;
+		}
 	}
 
 	copy_to_usernode(node, r_node);
@@ -789,7 +795,7 @@ static int do_cmd_set_votes(char *cmdbuf, int *retlen)
 		return -EINVAL;
 	}
 
-	recalculate_quorum(1);
+	recalculate_quorum(1, 0);
 
 	send_reconfigure(arg.nodeid, RECONFIG_PARAM_NODE_VOTES, arg.newvotes);
 
@@ -995,7 +1001,7 @@ static int do_cmd_register_quorum_device(char *cmdbuf, int *retlen)
         /* Keep this list valid so it doesn't confuse other code */
         list_init(&quorum_device->addr_list);
 
-	log_msg(LOG_INFO, "quorum device registered\n");
+	log_printf(LOG_INFO, "quorum device registered\n");
         return 0;
 }
 
@@ -1012,7 +1018,7 @@ static int do_cmd_unregister_quorum_device(char *cmdbuf, int *retlen)
 
         quorum_device = NULL;
 
-	log_msg(LOG_INFO, "quorum device unregistered\n");
+	log_printf(LOG_INFO, "quorum device unregistered\n");
         return 0;
 }
 
@@ -1020,19 +1026,19 @@ static void ccsd_timer_fn(void *arg)
 {
 	int ccs_err;
 
-	log_msg(LOG_DEBUG, "Polling ccsd for updated information\n");
+	log_printf(LOG_DEBUG, "Polling ccsd for updated information\n");
 	ccs_err = read_ccs_nodes(&config_version, 0);
 	if (ccs_err || config_version < wanted_config_version) {
-		log_msg(LOG_ERR, "Can't read CCS to get updated config version %d. Activity suspended on this node\n",
-			wanted_config_version);
+		log_printf(LOG_ERR, "Can't read CCS to get updated config version %d. Activity suspended on this node\n",
+				wanted_config_version);
 
 		openais_timer_add_duration((unsigned long long)ccsd_poll_interval*1000000, NULL,
 					   ccsd_timer_fn, &ccsd_timer);
 	}
 	else {
-		log_msg(LOG_ERR, "Now got CCS information version %d, continuing\n", config_version);
+		log_printf(LOG_ERR, "Now got CCS information version %d, continuing\n", config_version);
 		config_error = 0;
-		recalculate_quorum(0);
+		recalculate_quorum(0, 0);
 	}
 }
 
@@ -1043,11 +1049,12 @@ static void quorum_device_timer_fn(void *arg)
 	if (!quorum_device || quorum_device->state == NODESTATE_DEAD)
 		return;
 
+	P_MEMB("quorum_device_timer_fn\n");
 	gettimeofday(&now, NULL);
 	if (quorum_device->last_hello.tv_sec + quorumdev_poll/1000 < now.tv_sec) {
 		quorum_device->state = NODESTATE_DEAD;
-		log_msg(LOG_INFO, "lost contact with quorum device\n");
-		recalculate_quorum(0);
+		log_printf(LOG_INFO, "lost contact with quorum device\n");
+		recalculate_quorum(0, 0);
 	}
 	else {
 		openais_timer_add_duration((unsigned long long)quorumdev_poll*1000000, quorum_device,
@@ -1068,7 +1075,7 @@ static int do_cmd_poll_quorum_device(char *cmdbuf, int *retlen)
 		gettimeofday(&quorum_device->last_hello, NULL);
                 if (quorum_device->state == NODESTATE_DEAD) {
                         quorum_device->state = NODESTATE_MEMBER;
-                        recalculate_quorum(0);
+                        recalculate_quorum(0, 0);
 
 			openais_timer_add_duration((unsigned long long)quorumdev_poll*1000000, quorum_device,
 						   quorum_device_timer_fn, &quorum_device_timer);
@@ -1077,7 +1084,7 @@ static int do_cmd_poll_quorum_device(char *cmdbuf, int *retlen)
         else {
                 if (quorum_device->state == NODESTATE_MEMBER) {
                         quorum_device->state = NODESTATE_DEAD;
-                        recalculate_quorum(0);
+                        recalculate_quorum(0, 0);
 			openais_timer_delete(quorum_device_timer);
                 }
         }
@@ -1149,7 +1156,7 @@ static int do_cmd_get_node_addrs(char *cmdbuf, char **retbuf, int retsize, int *
 	int i;
 	char *outbuf = *retbuf + offset;
 	struct cl_get_node_addrs *addrs = (struct cl_get_node_addrs *)outbuf;
-	struct totem_ip_address node_ifs[num_interfaces];
+	struct totem_ip_address node_ifs[INTERFACE_MAX]; /* totempg_ifaces_get always copies INTERFACE_MAX addresses */
 	struct cluster_node *node;
 	char **status;
 
@@ -1187,6 +1194,7 @@ int process_command(struct connection *con, int cmd, char *cmdbuf,
 	int err = -EINVAL;
 	struct cl_version cnxman_version;
 	char *outbuf = *retbuf;
+	int value;
 
 	P_MEMB("command to process is %x\n", cmd);
 
@@ -1207,6 +1215,17 @@ int process_command(struct connection *con, int cmd, char *cmdbuf,
 		err = 0;
 		break;
 
+	case CMAN_CMD_SET_DIRTY:
+		us->flags |= NODE_FLAGS_DIRTY;
+		err = 0;
+		break;
+
+	case CMAN_CMD_SET_DEBUGLOG:
+		memcpy(&value, cmdbuf, sizeof(int));
+		set_debuglog(value);
+		err = 0;
+		break;
+
 	case CMAN_CMD_START_CONFCHG:
 		con->confchg = 1;
 		err = 0;
@@ -1464,20 +1483,20 @@ int our_nodeid()
 static int valid_transition_msg(int nodeid, struct cl_transmsg *msg)
 {
 	if (strcmp(msg->clustername, cluster_name) != 0) {
-		log_msg(LOG_ERR, "Node %d conflict, remote cluster name='%s', local='%s'\n",
+		log_printf(LOG_ERR, "Node %d conflict, remote cluster name='%s', local='%s'\n",
 			nodeid, msg->clustername, cluster_name);
 		return -1;
 	}
 
 	if (msg->cluster_id != cluster_id) {
-		log_msg(LOG_ERR, "Node %d conflict, remote cluster id=%d, local=%d\n",
+		log_printf(LOG_ERR, "Node %d conflict, remote cluster id=%d, local=%d\n",
 			nodeid, msg->cluster_id, cluster_id);
 		return -1;
 	}
 
 	if (msg->major_version != CNXMAN_MAJOR_VERSION) {
 
-		log_msg(LOG_ERR, "Node %d conflict, remote version id=%d, local=%d\n",
+		log_printf(LOG_ERR, "Node %d conflict, remote version id=%d, local=%d\n",
 			nodeid, msg->major_version, CNXMAN_MAJOR_VERSION);
 		return -1;
 	}
@@ -1489,7 +1508,7 @@ static int valid_transition_msg(int nodeid, struct cl_transmsg *msg)
 		ccs_err = read_ccs_nodes(&config_version, 0);
 		if (ccs_err || config_version < msg->config_version) {
 			config_error = 1;
-			log_msg(LOG_ERR, "Can't read CCS to get updated config version %d. Activity suspended on this node\n",
+			log_printf(LOG_ERR, "Can't read CCS to get updated config version %d. Activity suspended on this node\n",
 				msg->config_version);
 
 			wanted_config_version = msg->config_version;
@@ -1499,12 +1518,12 @@ static int valid_transition_msg(int nodeid, struct cl_transmsg *msg)
 		if (config_version > msg->config_version) {
 			// TODO tell everyone else to update...
 		}
-		recalculate_quorum(0);
+		recalculate_quorum(0, 0);
 	}
 
 
 	if (msg->config_version != config_version) {
-		log_msg(LOG_ERR, "Node %d conflict, remote config version id=%d, local=%d\n",
+		log_printf(LOG_ERR, "Node %d conflict, remote config version id=%d, local=%d\n",
 			nodeid, msg->config_version, config_version);
 		return -1;
 	}
@@ -1515,7 +1534,7 @@ static int valid_transition_msg(int nodeid, struct cl_transmsg *msg)
 
 void send_transition_msg(int last_memb_count, int first_trans)
 {
-	char buf[sizeof(struct cl_transmsg)+1024];
+	char buf[sizeof(struct cl_transmsg)+1024] __attribute__((aligned(8)));
 	struct cl_transmsg *msg = (struct cl_transmsg *)buf;
 	int len = sizeof(struct cl_transmsg);
 
@@ -1587,7 +1606,7 @@ static void byteswap_internal_message(char *data, int len)
 	case CLUSTER_MSG_KILLNODE:
 		killmsg = (struct cl_killmsg *)data;
 		killmsg->reason = swab16(killmsg->reason);
-		killmsg->nodeid = swab16(killmsg->nodeid);
+		killmsg->nodeid = swab32(killmsg->nodeid);
 		break;
 
 	case CLUSTER_MSG_LEAVE:
@@ -1636,21 +1655,21 @@ static void do_reconfigure_msg(void *data)
 				node->expected_votes = msg->value;
 			}
 		}
-		recalculate_quorum(1);  /* Allow decrease */
+		recalculate_quorum(1, 0);  /* Allow decrease */
 		break;
 
 	case RECONFIG_PARAM_NODE_VOTES:
 		node->votes = msg->value;
-		recalculate_quorum(1);  /* Allow decrease */
+		recalculate_quorum(1, 0);  /* Allow decrease */
 		break;
 
 	case RECONFIG_PARAM_CONFIG_VERSION:
 		if (read_ccs_nodes(&config_version, 0)) {
-			log_msg(LOG_ERR, "Can't read CCS to get updated config version %d. Activity suspended on this node\n",
+			log_printf(LOG_ERR, "Can't read CCS to get updated config version %d. Activity suspended on this node\n",
 				msg->value);
 
 			config_error = 1;
-			recalculate_quorum(0);
+			recalculate_quorum(0, 0);
 
 			wanted_config_version = config_version;
 			openais_timer_add_duration((unsigned long long)ccsd_poll_interval*1000000, NULL,
@@ -1704,9 +1723,33 @@ static void do_process_transition(int nodeid, char *data, int len)
 	msg->flags &= ~NODE_FLAGS_SEESDISALLOWED;
 
 	node = find_node_by_nodeid(nodeid);
+	if (!node) {
+		add_ais_node(nodeid, incarnation, num_ais_nodes);
+		node = find_node_by_nodeid(nodeid);
+	}
 	assert(node);
 
-        /* This is the killer. If the join_time of the node matches that already stored AND
+	/* Newer nodes 6.1.0 onwards, set the DIRTY flag if they have state. If the new node has been down
+	   and has state then we mark it disallowed because we cannot merge stateful nodes */
+	if (msg->flags & NODE_FLAGS_DIRTY && node->flags & NODE_FLAGS_BEENDOWN) {
+		/* Don't duplicate messages */
+		if (node->state != NODESTATE_AISONLY) {
+			if (cluster_is_quorate) {
+				P_MEMB("Killing node %s because it has rejoined the cluster with existing state", node->name);
+				log_printf(LOG_CRIT, "Killing node %s because it has rejoined the cluster with existing state", node->name);
+				node->state = NODESTATE_AISONLY;
+				send_kill(nodeid, CLUSTER_KILL_REJOIN);
+			}
+			else {
+				P_MEMB("Node %s not joined to cman because it has existing state", node->name);
+				log_printf(LOG_CRIT, "Node %s not joined to cman because it has existing state", node->name);
+				node->state = NODESTATE_AISONLY;
+			}
+		}
+		return;
+	}
+
+        /* This is for older nodes. If the join_time of the node matches that already stored AND
 	   the node has been down, then we kill it as this must be a rejoin */
 	if (msg->join_time == node->cman_join_time && node->flags & NODE_FLAGS_BEENDOWN) {
 		/* Don't duplicate messages */
@@ -1737,6 +1780,11 @@ static void do_process_transition(int nodeid, char *data, int len)
 		node->state = NODESTATE_AISONLY;
 
 	node->flags = msg->flags; /* This will clear the BEENDOWN flag of course */
+
+	/* Take into account any new expected_votes value that the new node has */
+	node->expected_votes = msg->expected_votes;
+	recalculate_quorum(0, 0);
+
 	if (node->fence_agent && msg->fence_agent[0] && strcmp(node->fence_agent, msg->fence_agent))
 	{
 		free(node->fence_agent);
@@ -1805,8 +1853,9 @@ static void process_internal_message(char *data, int len, int nodeid, int need_b
 
 	case CLUSTER_MSG_PORTSTATUS:
 		if (nodeid != us->node_id) {
-			P_MEMB("got PORTRESULT from %d, low bytes = %x %x\n", data[1], data[2]);
-			memcpy(node->port_bits, data+1, PORT_BITS_SIZE);
+			P_MEMB("got PORTRESULT from %d, low bytes = %x %x\n", nodeid, data[1], data[2]);
+			if (node)
+				memcpy(node->port_bits, data+1, PORT_BITS_SIZE);
 		}
 		break;
 
@@ -1819,7 +1868,8 @@ static void process_internal_message(char *data, int len, int nodeid, int need_b
 		killmsg = (struct cl_killmsg *)data;
 		P_MEMB("got KILL for node %d\n", killmsg->nodeid);
 		if (killmsg->nodeid == wanted_nodeid) {
-			log_msg(LOG_INFO, "cman killed by node %d because %s\n", nodeid,
+			/* Must use syslog directly here or the message will never arrive */
+			syslog(LOG_CRIT, "cman killed by node %d because %s\n", nodeid,
 				killmsg_reason(killmsg->reason));
 			exit(1);
 		}
@@ -1861,7 +1911,7 @@ static void process_internal_message(char *data, int len, int nodeid, int need_b
 		break;
 
 	default:
-		log_msg(LOG_WARNING, "Unknown protocol message %d received\n", msg->cmd);
+		log_printf(LOG_WARNING, "Unknown protocol message %d received\n", msg->cmd);
 		break;
 
 	}
@@ -1885,30 +1935,15 @@ void override_expected(int newexp)
 /* Add a node from CCS, note that it may already exist if user has simply updated the config file */
 void add_ccs_node(char *nodename, int nodeid, int votes, int expected_votes)
 {
-	struct totem_ip_address ipaddr;
-	struct cluster_node *node;
-
-	if (totemip_parse(&ipaddr, nodename, 0))
-	{
-		if (!nodeid) {
-			log_msg(LOG_ERR, "Error, can't find IP address and no nodeid for node %s - ignoring it\n", nodename);
-			return;
-		}
-		else {
-			log_msg(LOG_WARNING, "Warning, can't resolve IP address for node %s\n", nodename);
-			memset(&ipaddr, 0, sizeof(ipaddr));
-		}
-	}
-
 	/* Update node entry */
-	node = add_new_node(nodename, nodeid, votes, expected_votes, NODESTATE_DEAD);
+	add_new_node(nodename, nodeid, votes, expected_votes, NODESTATE_DEAD);
 }
 
 void add_ais_node(int nodeid, uint64_t incarnation, int total_members)
 {
 	struct cluster_node *node;
 
-	P_MEMB("add_ais_node ID=%d, incarnation = %d\n",nodeid, incarnation);
+	P_MEMB("add_ais_node ID=%d, incarnation = %lld\n",nodeid, incarnation);
 
 	node = find_node_by_nodeid(nodeid);
 	if (!node && total_members == 1) {
@@ -1919,22 +1954,11 @@ void add_ais_node(int nodeid, uint64_t incarnation, int total_members)
  	/* This really should exist!! */
 	if (!node) {
 		char tempname[256];
-		node = malloc(sizeof(struct cluster_node));
-		if (!node) {
-			log_msg(LOG_ERR, "error allocating node struct for id %d, but CCS doesn't know about it anyway\n",
-				nodeid);
-			return;
-		}
-		log_msg(LOG_ERR, "Got node from AIS id %d with no CCS entry\n", nodeid);
-
-		memset(node, 0, sizeof(struct cluster_node));
-		node_add_ordered(node);
-		node->state = NODESTATE_DEAD;
-		node->votes = 1;
+		log_printf(LOG_ERR, "Got node from AIS id %d with no CCS entry\n", nodeid);
 
 		/* Emergency nodename */
-		sprintf(tempname, "Node%d\n", nodeid);
-		node->name = strdup(tempname);
+		sprintf(tempname, "Node%d", nodeid);
+		node = add_new_node(tempname, nodeid, 1, total_members, NODESTATE_DEAD);
 	}
 
 	if (node->state == NODESTATE_DEAD) {
@@ -1942,7 +1966,7 @@ void add_ais_node(int nodeid, uint64_t incarnation, int total_members)
 		node->incarnation = incarnation;
 		node->state = NODESTATE_MEMBER;
 		cluster_members++;
-		recalculate_quorum(0);
+		recalculate_quorum(0, 0);
 	}
 }
 
@@ -1953,7 +1977,8 @@ void del_ais_node(int nodeid)
 	P_MEMB("del_ais_node %d\n", nodeid);
 
 	node = find_node_by_nodeid(nodeid);
-	assert(node);
+	if (!node)
+		return;
 
 	/* If the node was fenced while up (ie independantly of fenced) then
 	 * don't clear the fenced flag. There is a timeout associated with
@@ -1971,8 +1996,9 @@ void del_ais_node(int nodeid)
 	switch (node->state) {
 	case NODESTATE_MEMBER:
 		node->state = NODESTATE_DEAD;
+		memset(&node->port_bits, 0, sizeof(node->port_bits));
 		cluster_members--;
-		recalculate_quorum(0);
+		recalculate_quorum(0, 0);
 		break;
 
 	case NODESTATE_AISONLY:
@@ -1983,10 +2009,10 @@ void del_ais_node(int nodeid)
 		node->state = NODESTATE_DEAD;
 		cluster_members--;
 
-		if ((node->leave_reason & 0xF) == CLUSTER_LEAVEFLAG_REMOVED)
-			recalculate_quorum(1);
+		if ((node->leave_reason & 0xF) & CLUSTER_LEAVEFLAG_REMOVED)
+			recalculate_quorum(1, 1);
 		else
-			recalculate_quorum(0);
+			recalculate_quorum(0, 0);
 		break;
 
 	case NODESTATE_JOINING:
