@@ -24,7 +24,7 @@
 #include <netinet/in.h>
 
 #include "libdlm.h"
-/* #include "dlm_controld.h" */
+#include "dlm_controld.h"
 
 #define OPTION_STRING			"MhVvd:m:"
 
@@ -49,12 +49,12 @@ static void print_usage(void)
 {
 	printf("Usage:\n");
 	printf("\n");
-	printf("%s [options] [join|leave|lockdump|lockdebug]\n", prog_name);
+	printf("%s [options] [join|leave|lockdump|lockdebug|deadlock_check]\n", prog_name);
 	printf("\n");
 	printf("Options:\n");
 	printf("  -v               Verbose output\n");
 	printf("  -d <n>           Resource directory off/on (0/1), default 0\n");
-	printf("  -m               Permission mode for lockspace device (octal)\n");
+	printf("  -m <mode>        Permission mode for lockspace device (octal), default 0600\n");
 	printf("  -M               Print MSTCPY locks in lockdump (remote locks, locally mastered)\n");
 	printf("  -h               Print this help, then exit\n");
 	printf("  -V               Print program version information, then exit\n");
@@ -74,7 +74,7 @@ static void decode_arguments(int argc, char **argv)
 		switch (optchar) {
 		case 'm':
 			memset(modebuf, 0, sizeof(modebuf));
-			snprintf(modebuf, 8, optarg);
+			snprintf(modebuf, 8, "%s", optarg);
 			sscanf(modebuf, "%o", &create_mode);
 			break;
 
@@ -145,8 +145,6 @@ static void decode_arguments(int argc, char **argv)
 			operation = OP_LOCKDEBUG;
 			opt_ind = optind + 1;
 			break;
-		}
-#if 0
 		} else if (!strncmp(argv[optind], "spaces", 9) &&
 			   (strlen(argv[optind]) == 6)) {
 			operation = OP_SPACES;
@@ -159,7 +157,6 @@ static void decode_arguments(int argc, char **argv)
 			opt_ind = optind + 1;
 			break;
 		}
-#endif
 		optind++;
 	}
 
@@ -176,7 +173,6 @@ static void decode_arguments(int argc, char **argv)
 	}
 }
 
-#if 0
 static int do_write(int fd, void *buf, size_t count)
 {
 	int rv, off = 0;
@@ -195,16 +191,19 @@ static int do_write(int fd, void *buf, size_t count)
 	}
 	return 0;
 }
-#endif
 
 void do_join(char *name)
 {
 	dlm_lshandle_t *dh;
+	uint32_t flags = 0;
 
 	printf("Joining lockspace \"%s\", permission %o\n", name, create_mode);
 	fflush(stdout);
 
-	dh = dlm_new_lockspace(name, 0600, DLM_LSFL_NODIR);
+	if (!opt_dir)
+		flags = DLM_LSFL_NODIR;
+
+	dh = dlm_new_lockspace(name, create_mode, flags);
 	if (!dh) {
 		fprintf(stderr, "dlm_new_lockspace %s error %p %d\n",
 			name, dh, errno);
@@ -311,7 +310,7 @@ void do_lockdump(char *name)
 	int r_len;
 	int rv;
 	unsigned int time;
-	uint64_t	xid;
+	unsigned long long xid;
 	uint32_t	id;
 	int		nodeid;
 	uint32_t	remid;
@@ -368,6 +367,15 @@ void do_lockdump(char *name)
 			continue;
 		}
 
+		/* A hack because dlm-kernel doesn't set rqmode back to NL when
+		   a NOQUEUE convert fails, which means in a lockdump it looks
+		   like a granted lock is still converting since rqmode is not
+		   NL.  (does it make sense to include status in the output,
+		   e.g. G,C,W?) */
+
+		if (status == DLM_LKSTS_GRANTED)
+			rqmode = LKM_NLMODE;
+
 		printf("id %08x gr %s rq %s pid %u master %d \"%s\"\n",
 			id, mode_str(grmode), mode_str(rqmode),
 			ownpid, nodeid, r_name);
@@ -376,7 +384,6 @@ void do_lockdump(char *name)
 	fclose(file);
 }
 
-#if 0
 void do_spaces(void)
 {
 	/* TODO: get info from /sys/kernel/config/ */
@@ -428,7 +435,6 @@ static void do_deadlock_check(char *name)
 			strerror(errno));
 	close(fd);
 }
-#endif
 
 int main(int argc, char **argv)
 {
@@ -457,7 +463,7 @@ int main(int argc, char **argv)
 	case OP_LOCKDEBUG:
 		do_lockdebug(lsname);
 		break;
-#if 0
+
 	case OP_SPACES:
 		do_spaces();
 		break;
@@ -465,7 +471,6 @@ int main(int argc, char **argv)
 	case OP_DEADLOCK_CHECK:
 		do_deadlock_check(lsname);
 		break;
-#endif
 	}
 	return 0;
 }
