@@ -2,8 +2,8 @@
 *******************************************************************************
 **
 **  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
-**  
+**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
+**
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
 **  of the GNU General Public License v.2.
@@ -19,8 +19,11 @@
 #include <sys/wait.h>
 #include <string.h>
 #include <errno.h>
+#include <time.h>
 #include <syslog.h>
 
+#include <libcman.h>
+
 #include "ccs.h"
 
 #define MAX_METHODS		8
@@ -47,7 +50,7 @@ static void display_agent_output(char *agent, int fd)
 		strcat(msg, buf);
 
 		printf("%s\n", msg);
-		syslog(LOG_ERR, msg);
+		syslog(LOG_ERR, "%s", msg);
 
 		memset(buf, 0, sizeof(buf));
 		memset(msg, 0, sizeof(msg));
@@ -126,8 +129,9 @@ static int run_agent(char *agent, char *args)
 	return -1;
 }
 
-static int make_args(int cd, char *victim, char *method, int d, char *device,
-		     char **args_out) {
+static int make_args(int cd, char *victim, char *method, int d,
+		     char *device, char **args_out)
+{
 	char path[256], *args, *str;
 	int error;
 
@@ -249,7 +253,8 @@ static int count_devices(int cd, char *victim, char *method)
 	return i;
 }
 
-static int use_device(int cd, char *victim, char *method, int d, char *device)
+static int use_device(int cd, char *victim, char *method, int d,
+		      char *device)
 {
 	char path[256], *agent, *args = NULL;
 	int error;
@@ -264,7 +269,7 @@ static int use_device(int cd, char *victim, char *method, int d, char *device)
 	error = make_args(cd, victim, method, d, device, &args);
 	if (error)
 		goto out_agent;
-	
+
 	error = run_agent(agent, args);
 
 	free(args);
@@ -274,18 +279,75 @@ static int use_device(int cd, char *victim, char *method, int d, char *device)
 	return error;
 }
 
-int dispatch_fence_agent(int cd, char *victim)
+void update_cman(char *victim, char *method)
+{
+	cman_handle_t ch;
+	struct cman_node node;
+	uint64_t the_time = time(NULL);
+
+	ch = cman_admin_init(NULL);
+	if (!ch) {
+		syslog(LOG_ERR, "Unable to connect to to cman: %m");
+		return;
+	}
+	/* Convert name to a number */
+	memset(&node, 0, sizeof(node));
+	strcpy(node.cn_name, victim);
+
+	/* Mark it as fenced */
+	if (!cman_get_node(ch, 0, &node))
+		cman_node_fenced(ch, node.cn_nodeid, the_time, method);
+	else
+		syslog(LOG_ERR, "can't get node number for node %s\n", victim);
+	cman_finish(ch);
+}
+
+int dispatch_fence_agent(char *victim, int force)
 {
 	char *method = NULL, *device = NULL;
-	int num_methods, num_devices, m, d, error = -1;
+	char *victim_nodename = NULL;
+	int num_methods, num_devices, m, d, error = -1, cd;
+
+	if (force)
+		cd = ccs_force_connect(NULL, 0);
+	else {
+		while ((cd = ccs_connect()) < 0)
+			sleep(1);
+	}
+
+	if (cd < 0) {
+		syslog(LOG_ERR, "cannot connect to ccs %d\n", cd);
+		return -1;
+	}
+
+	if (ccs_lookup_nodename(cd, victim, &victim_nodename) == 0)
+		victim = victim_nodename;
 
 	num_methods = count_methods(cd, victim);
 
 	for (m = 0; m < num_methods; m++) {
 
 		error = get_method(cd, victim, m, &method);
+
+		/* if the connection timed out while we were trying 
+		 * to fence, try to open the connection again
+		 */
+		if (error == -EBADR) {
+			syslog(LOG_INFO, "ccs connection timed out, "
+				"retrying\n");
+
+			while ((cd = ccs_connect()) < 0)
+				sleep(1);
+			
+			error = get_method(cd, victim, m, &method);
+
 		if (error)
 			continue;
+		} else if (error)
+			continue;
+
+		/* if num_devices is zero we should return an error */
+		error = -1;
 
 		num_devices = count_devices(cd, victim, method);
 
@@ -298,18 +360,23 @@ int dispatch_fence_agent(int cd, char *victim)
 			if (error)
 				break;
 
+			update_cman(victim, device);
 			free(device);
 			device = NULL;
 		}
 
 		if (device)
 			free(device);
+		if (victim_nodename)
+			free(victim_nodename);
 		free(method);
 
 		if (!error)
 			break;
 	}
 
+	ccs_disconnect(cd);
+
 	return error;
 }
 
