@@ -2,7 +2,7 @@
 *******************************************************************************
 **
 **  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
 **  
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -37,29 +37,33 @@
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#include <sys/poll.h>
 
-
-#include "cnxman-socket.h"
 #include "list.h"
+#include "libgroup.h"
 
+#define MAX_NODENAME_LEN	255   /* should match libcman.h */
+#define MAX_GROUPNAME_LEN	32    /* should match libgroup.h */
+#define MAX_NODES		256
+#define MAXARGS                 100  /* FIXME */
+#define MAXLINE                 256
+#define MAX_CLIENTS		5
+#define DUMP_SIZE               (1024 * 1024)
 
-extern char *             prog_name;
-extern int                fenced_debug;
-extern int                debug_sock;
-extern char               debug_buf[256];
-extern struct sockaddr_un debug_addr;
-extern socklen_t          debug_addrlen;
-
-
-#define FENCED_SOCK_PATH "fenced_socket"
+#define DEFAULT_POST_JOIN_DELAY	6
+#define DEFAULT_POST_FAIL_DELAY	0
+#define DEFAULT_CLEAN_START	0
 #define DEFAULT_OVERRIDE_PATH	"/var/run/cluster/fenced_override"
+#define FENCED_SOCK_PATH	"fenced_socket"
 
-#define DEFAULT_POST_JOIN_DELAY   6
-#define DEFAULT_POST_FAIL_DELAY   0
-#define DEFAULT_CLEAN_START       0
+extern char			*prog_name;
+extern int			daemon_debug_opt;
+extern char			daemon_debug_buf[256];
+extern char dump_buf[DUMP_SIZE];
+extern int dump_point;
+extern int dump_wrap;
 
-/* should match service.h MAX_SERVICE_NAME_LEN */
-#define MAX_NAME_LEN	33
+extern void daemon_dump_save(void);
 
 /* use this one before we fork into the background */
 #define die1(fmt, args...) \
@@ -81,7 +85,7 @@ do \
 } \
 while (0)
 
-#define FENCE_ASSERT(x, todo) \
+#define ASSERT(x, todo) \
 do \
 { \
   if (!(x)) \
@@ -102,19 +106,19 @@ for (;;) \
   sleep(1); \
 }
 
-/* log_debug messages only appear when -D is used and then they go to stdout */
-/* #define log_debug(fmt, args...) printf("fenced: " fmt "\n", ##args) */
-
 #define log_debug(fmt, args...) \
-do \
-{ \
-	snprintf(debug_buf, 255, "%ld " fmt "\n", time(NULL), ##args); \
-	if (fenced_debug) printf("fenced: %s", debug_buf); \
-	sendto(debug_sock, debug_buf, strlen(debug_buf), 0, \
-	       (struct sockaddr *)&debug_addr, debug_addrlen); \
-} \
-while (0)
-	
+do { \
+	snprintf(daemon_debug_buf, 255, "%ld " fmt "\n", time(NULL), ##args); \
+	if (daemon_debug_opt) fprintf(stderr, "%s", daemon_debug_buf); \
+	daemon_dump_save(); \
+} while (0)
+
+#define log_error(fmt, args...) \
+do { \
+	log_debug(fmt, ##args); \
+	syslog(LOG_ERR, fmt, ##args); \
+} while (0)
+
 
 struct fd;
 struct fd_node;
@@ -126,16 +130,17 @@ typedef struct commandline commandline_t;
 
 struct commandline
 {
-	char name[MAX_NAME_LEN];
-	int debug;
 	int post_join_delay;
 	int post_fail_delay;
+	int override_time;
+	int pad;
 	char *override_path;
 	int8_t clean_start;
 	int8_t post_join_delay_opt;
 	int8_t post_fail_delay_opt;
 	int8_t clean_start_opt;
 	int8_t override_path_opt;
+	int8_t override_time_opt;
 };
 
 #define FDFL_RUN        (0)
@@ -143,39 +148,53 @@ struct commandline
 #define FDFL_FINISH     (2)
 
 struct fd {
-	struct commandline	*comline;
-	int			cl_sock;
-	uint32_t 		our_nodeid;
-	uint32_t 		local_id;	/* local unique fd ID */
-	uint32_t 		global_id;	/* global unique fd ID */
-
+	struct list_head	list;
+	int			global_id;	/* global unique fd ID */
 	int 			last_stop;
 	int 			last_start;
 	int 			last_finish;
-
 	int			first_recovery;
 	int 			prev_count;
+	int			leave;
 	struct list_head 	prev;
 	struct list_head 	victims;
 	struct list_head 	leaving;
 	struct list_head	complete;
-
-	int 			namelen;
-	char 			name[1];
+	char 			name[MAX_GROUPNAME_LEN+1];
 };
 
 struct fd_node {
 	struct list_head 	list;
-	uint32_t 		nodeid;
-	int 			namelen;
-	char 			name[1];
+	int			nodeid;
+	char 			name[MAX_NODENAME_LEN+1];
 };
 
 
-void add_complete_node(fd_t *fd, uint32_t nodeid, uint32_t len, char *name);
-void do_recovery(fd_t *fd, struct cl_service_event *ev,
-		 struct cl_cluster_node *cl_nodes);
+/* main.c */
+fd_t *find_domain(char *name);
+
+/* recover.c */
+void add_complete_node(fd_t *fd, int nodeid, char *name);
+void do_recovery(fd_t *fd, int start_type, int member_count, int *nodeids);
 void do_recovery_done(fd_t *fd);
-int dispatch_fence_agent(int cd, char *victim);
+
+/* agent.c */
+int dispatch_fence_agent(char *victim, int force);
+
+/* group.c */
+int setup_groupd(void);
+void exit_groupd(void);
+int process_groupd(void);
+int in_groupd_cpg(int nodeid);
+
+/* member_xxx.c */
+int setup_member(void);
+int process_member(void);
+void exit_member(void);
+int update_cluster_members(void);
+int is_member(char *name);
+int is_fenced(char *name);
+fd_node_t *get_new_node(fd_t *fd, int nodeid, char *in_name);
+void update_cman(char *victim, char *agent);
 
 #endif				/*  __FD_DOT_H__  */
