@@ -14,7 +14,7 @@
 #ifndef __GFS_DOT_H__
 #define __GFS_DOT_H__
 
-#define GFS_RELEASE_NAME "2.01.00"
+#define RELEASE_VERSION "2.02.00"
 
 #include <linux/lm_interface.h>
 
