@@ -2,7 +2,7 @@
 *******************************************************************************
 **
 **  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2005 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -36,63 +36,6 @@ struct dir_status {
 };
 
 
-static int check_leaf(struct fsck_inode *ip, uint64_t block, osi_buf_t **lbh,
-		      void *private)
-{
-	uint64_t chain_no;
-	struct fsck_sb *sbp = ip->i_sbd;
-	struct gfs_leaf leaf;
-	osi_buf_t *chain_head = NULL;
-	osi_buf_t *bh = NULL;
-	int chain=0;
-	int error;
-
-	chain_no = block;
-
-	do {
-		/* FIXME: check the range of the leaf? */
-		/* check the leaf and stuff */
-
-		error = get_and_read_buf(sbp, chain_no, &bh, 0);
-
-		if(error){
-			stack;
-			goto fail;
-		}
-
-		gfs_leaf_in(&leaf, BH_DATA(bh));
-
-		/* Check the leaf headers */
-
-		if(!chain){
-			chain = 1;
-			chain_head = bh;
-			chain_no = leaf.lf_next;
-		}
-		else {
-			relse_buf(sbp, bh);
-			bh = NULL;
-			break;
-		}
-	} while(chain_no);
-
-	*lbh = chain_head;
-	return 0;
-
- fail:
-	/* FIXME: check this error path */
-	if(chain_head){
-		if(chain_head == bh){ bh = NULL; }
-		relse_buf(sbp, chain_head);
-	}
-	if(bh)
-		relse_buf(sbp, bh);
-
-	return -1;
-
-}
-
-
 /* Set children's parent inode in dir_info structure - ext2 does not set
  * dotdot inode here, but instead in pass3 - should we? */
 int set_parent_dir(struct fsck_sb *sbp, uint64_t childblock,
@@ -277,10 +220,9 @@ int check_dentry(struct fsck_inode *ip, struct gfs_dirent *dent,
 			tmp_name);
 		if(query(ip->i_sbd, "Clear directory entry tp out of range block? (y/n) ")) {
 			log_err("Clearing %s\n", tmp_name);
-			if(dirent_del(ip, bh, prev_de, dent)) {
-				stack;
-				return -1;
-			}
+			if(dirent_del(ip, bh, prev_de, dent))
+				log_err("Error encountered while removing bad "
+					"directory entry.  Skipping.\n");
 			return 1;
 		} else {
 			log_err("Directory entry to out of range block remains\n");
@@ -329,7 +271,8 @@ int check_dentry(struct fsck_inode *ip, struct gfs_dirent *dent,
 	   q.block_type != inode_lnk && q.block_type != inode_blk &&
 	   q.block_type != inode_chr && q.block_type != inode_fifo &&
 	   q.block_type != inode_sock) {
-		log_err("Found directory entry '%s' in %"PRIu64" to something"
+		log_err("Found directory entry '%s' in block %"
+			PRIu64" to something"
 			" not a file or directory!\n", tmp_name,
 			ip->i_num.no_addr);
 		log_debug("block #%"PRIu64" in %"PRIu64"\n",
@@ -339,10 +282,9 @@ int check_dentry(struct fsck_inode *ip, struct gfs_dirent *dent,
 			/* FIXME: make sure all blocks referenced by
 			 * this inode are cleared in the bitmap */
 
-			if(dirent_del(ip, bh, prev_de, dent)) {
-				stack;
-				return -1;
-			}
+			if(dirent_del(ip, bh, prev_de, dent))
+				log_err("Error encountered while removing bad "
+					"directory entry.  Skipping.\n");
 			log_warn("Directory entry '%s' cleared\n", tmp_name);
 			return 1;
 		} else {
@@ -369,10 +311,9 @@ int check_dentry(struct fsck_inode *ip, struct gfs_dirent *dent,
 			check_inode_eattr(entry_ip, &clear_eattrs);
 			free_inode(&entry_ip);
 
-			if(dirent_del(ip, bh, prev_de, dent)) {
-				stack;
-				return -1;
-			}
+			if(dirent_del(ip, bh, prev_de, dent))
+				log_err("Error encountered while removing bad "
+					"directory entry.  Skipping.\n");
 			return 1;
 		} else {
 			log_err("Stale directory entry remains\n");
@@ -561,7 +502,7 @@ int check_dentry(struct fsck_inode *ip, struct gfs_dirent *dent,
 
 struct metawalk_fxns pass2_fxns = {
 	.private = NULL,
-	.check_leaf = check_leaf,
+	.check_leaf = NULL,
 	.check_metalist = NULL,
 	.check_data = NULL,
 	.check_eattr_indir = check_eattr_indir,
@@ -808,9 +749,14 @@ int pass2(struct fsck_sb *sbp, struct options *opts)
 	}
 	int error = 0;
 
+	log_info("Checking directory inodes.\n");
 	/* Grab each directory inode, and run checks on it */
 	for(i = 0; i < sbp->last_fs_block; i++) {
 
+		warm_fuzzy_stuff(i);
+		if (skip_this_pass || fsck_abort) /* if asked to skip the rest */
+			return 0;
+
 		/* Skip the root inode - it's checked above */
 		if(i == sbp->sb.sb_root_di.no_addr)
 			continue;
@@ -824,7 +770,7 @@ int pass2(struct fsck_sb *sbp, struct options *opts)
 		if(q.block_type != inode_dir)
 			continue;
 
-		log_info("Checking directory inode at block %"PRIu64"\n", i);
+		log_debug("Checking directory inode at block %"PRIu64"\n", i);
 
 
 		memset(&ds, 0, sizeof(ds));
